#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

import wx
import os

wx.SetDefaultPyEncoding("iso-8859-15")

APPDIR = os.path.dirname(os.path.abspath(__file__))
HINTERGRUNDBILD = os.path.join(APPDIR, "images", "hintergrund_01.png")
SPIELFELDBILD = os.path.join(APPDIR, "images", "spielfeld_02.png")
KUGELBILD = os.path.join(APPDIR, "images", "kugel_01.png")


class SpielfeldPanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.bmp = self.generate_bitmap()
        self.Bind(wx.EVT_PAINT, self.on_paint)
        self.SetClientSize(self.bmp.GetSize())
        self.SetSizeHintsSz(self.GetSize())
        self.Fit()
    
    
    def generate_bitmap(self):
        bg_bmp = wx.Bitmap(HINTERGRUNDBILD)
        fg_bmp = wx.Bitmap(SPIELFELDBILD)
        
        bmp = wx.EmptyBitmap(*bg_bmp.GetSize())
        
        mdc = wx.MemoryDC(bmp)
        mdc.DrawBitmap(bg_bmp, 0, 0)
        mdc.DrawBitmap(fg_bmp, 0, 0)
        
        mdc.SelectObject(wx.NullBitmap)
        return bmp
    

    def on_paint(self, event):
        dc = wx.BufferedPaintDC(self, self.bmp)


class KugelWindow(wx.Window):
    
    def __init__(self, parent, spielfeld_bmp):
        wx.Window.__init__(self, parent)
        
        self.spielfeld_bmp = spielfeld_bmp
        self.kugel_bmp = wx.Bitmap(KUGELBILD)
        self.bmp = self.generate_bitmap()
        self.SetClientSize(self.kugel_bmp.GetSize())
        self.SetSizeHintsSz(self.GetSize())
        self.Fit()
        
        self.Bind(wx.EVT_LEFT_DOWN, self.on_left_down)
        self.Bind(wx.EVT_LEFT_UP, self.on_left_up)
        self.Bind(wx.EVT_MOTION, self.on_motion)
        self.Bind(wx.EVT_PAINT, self.on_paint)
        self.Bind(wx.EVT_MOVE, self.paint_bitmap)
    
    
    def generate_bitmap(self):
        
        w, h = tuple(self.kugel_bmp.GetSize())
        
        bmp = wx.EmptyBitmap(w, h)
        dest_dc = wx.MemoryDC(bmp)
        
        x, y = tuple(self.GetPosition())
        
        src_dc = wx.MemoryDC(self.spielfeld_bmp)
        dest_dc.Blit(0, 0, w, h, src_dc, x, y)
        dest_dc.DrawBitmap(self.kugel_bmp, 0, 0)
        
        dest_dc.SelectObject(wx.NullBitmap)
        return bmp
    
    
    def paint_bitmap(self, event = None):
        self.bmp = self.generate_bitmap()
        dc = wx.ClientDC(self)
        dc.DrawBitmap(self.bmp, 0, 0)
    
    
    def on_paint(self, event):
        dc = wx.BufferedPaintDC(self, self.bmp)
    

    def on_left_down(self, evt):
        self.CaptureMouse()
        x, y = self.ClientToScreen(evt.GetPosition())
        originx, originy = self.GetPosition()
        dx = x - originx
        dy = y - originy
        self.mp_delta = ((dx, dy))


    def on_left_up(self, evt):
        if self.HasCapture():
            self.ReleaseMouse()


    def on_motion(self, evt):
        if evt.Dragging() and evt.LeftIsDown():
            x, y = self.ClientToScreen(evt.GetPosition())
            fp = (x - self.mp_delta[0], y - self.mp_delta[1])
            self.Move(fp)     
    

class MyFrame(wx.Frame):
    
    def __init__(self, parent = None, title = "Spiel"):
        wx.Frame.__init__(self, parent, -1, title)
        
        spielfeld_panel = SpielfeldPanel(self)
        spielfeld_bmp = spielfeld_panel.bmp
        
        kugel1 = KugelWindow(spielfeld_panel, spielfeld_bmp)
        
        self.Fit()
        self.SetSizeHintsSz(self.GetSize())
    

class MyApp(wx.PySimpleApp):
    
    def OnInit(self):
        
        frame = MyFrame()
        frame.Center()
        frame.Show()
        
        return True


def main():
    app = MyApp()
    app.MainLoop()


if __name__ == "__main__":
    main()
